/*
    File:       ARMlet.cpp  

    Summary:    Tapwave Native Application entrypoint. It also defines
                hooks for relocation support, floating point, and
                global constructors.

    Author:     Hong Zhang

    Copyright:  Copyright  2003 by Tapwave, Inc.

    Disclaimer: IMPORTANT: This Tapwave software is provided by
                Tapwave, Inc. ("Tapwave").  Your use is subject to and
                governed by terms and conditions of the Software
                Development Kit Agreement ("SDK Agreement") between
                you and Tapwave.  If you have not entered into
                Tapwaves standard SDK Agreement with Tapwave, you
                have no right or license to use, reproduce, modify,
                distribute or otherwise exploit this Tapwave software.
                You may obtain a copy of Tapwaves standard SDK
                Agreement by calling 650-960-1817 or visiting Tapwave
                at http://www.tapwave.com/developers/.

    Change History (most recent first):
        030702  Update comment regarding Tapwave Native Application.
        030805  Update to provide basic time support for MSL.
*/

#include <TapWave.h>

#ifndef __PALMOS_ARMLET__
#error "__PALMOS_ARMLET__ is not defined!!!"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/*
 * This variable holds the tapwave armlet interface dispatch
 * table. General ARM API calls will go through this table.
 */
struct TapWaveGlue* twGlue;

#ifdef __MWERKS__

#include <string.h>
#include <ctype.h>
#include <float.h>
#include <time.h>
#include <file_struc.h>
#include <console_io.h>

using namespace std;

/*
 * These symbols aren't real, but are linker-generated symbols that
 * mark the start and end of the various data sections.
 */
extern long __DataStart__[];
extern long __RODataStart__[];
extern long __BSSStart__[];
extern long __BSSEnd__[];
extern long __CodeRelocStart__[];
extern long __CodeRelocEnd__[];
extern long __DataRelocStart__[];
extern long __DataRelocEnd__[];

/*
 * This function performs relocation for Tapwave Native Application.
 */
static void relocate(void)
{
    // this symbol points to the very beginning of current application
    long base = (long) __ARMlet_Startup__;
    long *cur, *end;

    // handle code-to-data relocation
    cur = __CodeRelocStart__;
    end = __CodeRelocEnd__;
    for (; cur < end; cur++) {
        *(long*)(base + *cur) += base;
    }

    // handle data-to-data relocation
    cur = __DataRelocStart__;
    end = __DataRelocEnd__;
    for (; cur < end; cur++) {
        *(long*)(base + *cur) += base;
    }
}

/*
 * The following functions provide malloc/free support to Metrowerks
 * Standard Library (MSL). This feature requires the MSL library be
 * built with _MSL_OS_DIRECT_MALLOC enabled.
 */
void*
__sys_alloc(size_t size)
{
    return MemPtrNew(size);
}

void
__sys_free(void* ptr)
{
    (void) MemPtrFree(ptr);
}

size_t
__sys_pointer_size(void* ptr)
{
    return (size_t) MemPtrSize(ptr);
}

#if 0
/*
 * The following functions provide hooks for stdin/stdout/stderr.
 * Applications may override them to do whatever they want. The
 * default implementation redirects console IO through ARM semihosting
 * interface.
 */
int
__close_console(__file_handle file)
{
    return __no_io_error;
}

int
__read_console(__file_handle file, unsigned char * buff, size_t * count, __ref_con ref_con)
{
    return __io_EOF;
}

int
__write_console(__file_handle file, unsigned char * buff, size_t * count, __ref_con ref_con)
{
    return __no_io_error;
}
#endif

/*
 * The following functions provide stdio support to Metrowerks
 * Standard Library (MSL). This feature requires the MSL library be
 * built with _No_Disk_File_OS_Support disabled.
 */
int
__open_file(const char * name, __file_modes mode, __file_handle * handle)
{
    UInt16 volume = 0;
    UInt16 vfsmode = 0;
    Err err = 0xFFFF;
    
    if (isalpha(name[0]) && name[1] == ':') {
        volume = (name[0] & 0x1F);
        name += 2;
    }
    if (mode.open_mode == __create_if_necessary) {
        vfsmode |= vfsModeCreate;
    } else if (mode.open_mode == __create_or_truncate) {
        vfsmode |= vfsModeCreate | vfsModeTruncate;
    }
    if (mode.io_mode == __read) {
        vfsmode |= vfsModeRead;
    } else if (mode.io_mode == __write) {
        vfsmode |= vfsModeWrite;
    } else if (mode.io_mode == __read_write) {
        vfsmode |= vfsModeReadWrite;
    } else if (mode.io_mode == __append) {
        vfsmode |= vfsModeWrite;
    }
    if (volume != 0) {
        int slot = (volume == 3 ? twSlotInternal : volume);
        for (UInt32 it = vfsIteratorStart; it != vfsIteratorStop; ) {
            // iterate all volumes
            if (VFSVolumeEnumerate(&volume, &it)) {
                break;
            }
            if (slot == TwGetSlotNumberForVolume(volume)) {
                err = VFSFileOpen(volume, name, vfsmode, handle);
                break;
            }
        }
    } else {
        for (UInt32 it = vfsIteratorStart; it != vfsIteratorStop; ) {
            // iterate all volumes
            if (VFSVolumeEnumerate(&volume, &it)) {
                break;
            }
            err = VFSFileOpen(volume, name, vfsmode, handle);
            if (err == errNone) {
                break;
            }
        }
    }
    if (err == errNone && mode.io_mode == __append) {
        (void) VFSFileSeek((FileRef) *handle, vfsOriginEnd, 0);
    }
    return (err ? __io_error : __no_io_error);
}

int
__close_file(__file_handle file)
{
    Err err = VFSFileClose((FileRef) file);
    return (err ? __io_error : __no_io_error);
}

int
__read_file(__file_handle file, unsigned char * buff, size_t * count, __ref_con ref_con)
{
    Err err = VFSFileRead((FileRef) file, *count, buff, count);
    return (err ? (err == vfsErrFileEOF ? __io_EOF : __io_error) : __no_io_error);
}

int
__write_file(__file_handle file, unsigned char * buff, size_t * count, __ref_con ref_con)
{
    Err err = VFSFileWrite((FileRef) file, *count, buff, count);
    return (err ? __io_error : __no_io_error);
}

int
__position_file(__file_handle file, fpos_t * position, int mode, __ref_con ref_con)
{
    Err err = VFSFileSeek((FileRef) file, (FileOrigin) mode, (Int32) *position);
    if (err == errNone) {
        err = VFSFileTell((FileRef) file, position);
    }
    return (err ? __io_error : __no_io_error);
}

/*
 * The following functions provide simple time support to MSL.
 */
time_t
__get_time(void)
{
    // Convert epoch 1904-01-01 to 1970-01-01
    return TimGetSeconds() - 2082844800;
}

int
__to_gm_time(time_t* time)
{
    // Convert local time to utc time
    *time -= (PrefGetPreference(prefTimeZone) + PrefGetPreference(prefDaylightSavingAdjustment)) * 60;
    return 1;
}

int
__isdst(void)
{
    // Check if we in day light saving time currently.
    return PrefGetPreference(prefDaylightSavingAdjustment) != 0;
}

/*
 * This is the real entrypoint for Tapwave Native Application. It
 * depends on various CodeWarrior 9.2 compiler/linker/runtime features.
 */
static ARMLET_CALLBACK UInt32
Startup(const void *emulStateP, void *userData68KP, Call68KFuncType *call68KFuncP)
{
    if (emulStateP) {
        // COMMENT: normal pace native object launch
        return PilotMain(sysAppLaunchCmdNormalLaunch, userData68KP, 0);
    } else {
        // Setup armlet interface dispatch table
        twGlue = (struct TapWaveGlue*) userData68KP;

        // OPTIONAL: relocate data segment
        relocate();

        // OPTIONAL: initialize the floating-point library
        _fp_init();

        return 0;
    }
}

asm UInt32
__ARMlet_Startup__(const void *emulStateP, void *userData68KP, Call68KFuncType *call68KFuncP)
{
    b        Startup             //   0 forwarding call
    nop                          //   4
    nop                          //   8
    nop                          //  12
    nop                          //  16
    nop                          //  20
    nop                          //  24
    dcd      'TWNA'              //  28 Tapwave Native Application
    dcd      twGlue              //  32
    dcd      'cdwr'              //  36 make this section the same as the ARMlet_Startup
    dcd      __DataStart__       //  40 used by CodeWarrior 9.2
    dcd      __RODataStart__     //  44
    dcd      __BSSStart__        //  48
    dcd      __BSSEnd__          //  52
    dcd      __CodeRelocStart__  //  56
    dcd      __CodeRelocEnd__    //  60
    dcd      __DataRelocStart__  //  64
    dcd      __DataRelocEnd__    //  68
}

#endif // __MWERKS__

#ifdef _WIN32
/*
 * This is the real entrypoint for Tapwave Native Application.
 * It is a DLL export function, whose name is referred by Application.rcp
 */
ARMLET_CALLBACK UInt32
__ARMlet_Startup__(const void *emulStateP, void *userData68KP, Call68KFuncType *call68KFuncP)
{
    // Special call for initialization
    if (!emulStateP) {
        /* Setup armlet interface dispatch table */
        twGlue = (struct TapWaveGlue*) userData68KP;

        return 0;
    }

    return PilotMain(sysAppLaunchCmdNormalLaunch, NULL, 0);
}
#endif // _WIN32

#ifdef __cplusplus
} // extern "C"
#endif
